#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

require "al_worker_ipc"
require "al_worker_message"
require "al_worker_timer"
require "al_worker_debug"


class KitchenTimer < AlWorker

  def initialize()
    super( "kitchen_timer" )
  end


  ##
  # イニシャライザ
  #
  def initialize2()
    @timer_counter = 0
    @input_str = ""
    set_state( "INPUT" )
    set_value( "display_digit", format_count(@timer_counter) )

    AlWorker.log.level = Logger::DEBUG
    @ipc = Ipc.new()
    @ipc.chmod = 0666
    @ipc.run( self )
    @timer = Timer.periodic( 1 )
    @msg = NumberedMessage.new()
    Debug.run( self )
  end


  ##
  # 表示フォーマッティング
  #
  def format_count( n )
    case n
    when Numeric
      return sprintf( "%02d:%02d", n / 60, n % 60 )

    when String
      s = "0000" + n
      return "#{s[-4,2]}:#{s[-2,2]}"
    end
  end


  ##
  # (IPC) クライアント（ブラウザ, JavaScript）からのリクエストを受け、
  # メッセージ（イベント）をServerSentEventsプロトコルで返す。
  #
  def ipc_a_listen_ssev( sock, param )
    tid = param["LAST_EVENT_ID"] == 0 ? @msg.tid : param["LAST_EVENT_ID"]

    @msg.cycle( tid + 1 ) { |m|
      sock.puts "id: #{m[:TID].to_i}"
      sock.puts "data: #{m.to_json}"
      sock.puts ""
    }
  end


  ##
  # (IPC) 数字キーが押された場合
  #
  def ipc_digit( sock, param )
    digit = param["value"]
    trigger_event( "DIGIT", digit )
    reply( sock, 200, "OK" )
  end


  ##
  # (IPC) 開始が押された
  #
  def ipc_start( sock, param )
    trigger_event( "START" )
    reply( sock, 200, "OK" )
  end


  ##
  # (IPC) 停止が押された
  #
  def ipc_stop( sock, param )
    trigger_event( "STOP" )
    reply( sock, 200, "OK" )
  end


  ##
  # 数字入力
  #
  def state_INPUT_event_DIGIT( digit )
    @input_str << digit.to_s
    set_value( "display_digit", format_count(@input_str) )
    @msg.send( :value=>format_count(@input_str) )
  end


  ##
  # カウント開始
  #
  def state_INPUT_event_START()
    s = "0000#{@input_str}"
    @timer_counter = s[-4,2].to_i * 60 + s[-2,2].to_i
    return  if @timer_counter == 0
    @input_str = ""

    @timer.run() { timer_work() }
    next_state( "COUNTDOWN" )
  end


  ##
  # 入力値クリア
  #
  def state_INPUT_event_STOP()
    @input_str = ""
    set_value( "display_digit", format_count(@input_str) )
    @msg.send( :value=>format_count(@input_str) )
  end


  na :state_INPUT_event_TIMER
  na :state_INPUT_event_TIMEUP

  na :state_COUNTDOWN_event_DIGIT
  na :state_COUNTDOWN_event_START

  ##
  # カウント中断
  #
  def state_COUNTDOWN_event_STOP()
    @timer.stop()
    set_value( "display_digit", format_count(0) )
    @msg.send( :value=>format_count(0) )
    next_state( "INPUT" )
  end


  ##
  # カウントダウン中　表示更新
  #
  def state_COUNTDOWN_event_TIMER()
    set_value( "display_digit", format_count(@timer_counter) )
    @msg.send( :value=>format_count(@timer_counter) )
  end


  ##
  # タイムアップ
  #
  def state_COUNTDOWN_event_TIMEUP()
    set_value( "display_digit", "TIMEUP!" )
    @msg.send( :value => "TIMEUP!" )
    @msg.send( :sound => "play" )
    next_state( "ALERM" )
  end


  na :state_ALERM_event_DIGIT
  na :state_ALERM_event_START


  ##
  # アラーム停止
  #
  def state_ALERM_event_STOP()
    @timer.stop()
    set_value( "display_digit", format_count(0) )
    @msg.send( :value=>format_count(0) )
    next_state( "INPUT" )
  end


  ##
  # アラームを鳴らす
  #
  def state_ALERM_event_TIMER()
    @msg.send( :sound => "play" )
  end


  na :state_ALERM_event_TIMEUP


  ##
  # タイマーによる1秒ごとのワーク
  #
  def timer_work()
    @timer_counter -= 1
    if @timer_counter == 0
      trigger_event( "TIMEUP" )
    else
      trigger_event( "TIMER" )
    end
  end
end

KitchenTimer.new.daemon
